#ifndef F_CPU
	#define F_CPU 12000000
#endif

#include <avr/interrupt.h>
#include "i2clcd.h"

#define PRELOAD 55000
#define ADDR_LM75 	0x92

volatile int i = 0;

volatile unsigned char h_byte;	// Hight-Byte Temperatur-Registers LM75
volatile unsigned char l_byte;	// Low-Byte Temperatur-Registers LM75


// ISR Timer1-Ueberlauf
ISR(TIMER1_OVF_vect)
{
	TCNT1 = PRELOAD;						// Preloadwert setzen
	i++;
}


void lm75_init(void)
{
	i2c_start(ADDR_LM75+I2C_WRITE);
	i2c_write(0);							// Funktion Temperatur lesen
	i2c_stop();
	lcd_wait_ms(100);						// etwas warten bevor es weiter geht...
}


void lm75_read(void)
{
	i2c_start(ADDR_LM75+I2C_READ);
	h_byte = i2c_readAck();
	l_byte = i2c_readNak();
	i2c_stop();
}


int main(void)
{
	int old_i = 0;
	char s[10];
	unsigned char x, y;
	
	// Timer initialisieren...
    	TCNT1   = PRELOAD;			// Preloadwert setzen
	TCCR1B |= (1 << CS02) | (1 << CS00);	// Prescaler Timer1 auf 1024 stellen
	TIMSK  |= (1 << TOIE1);			// Timer1 Overflow Interrupt enable
	sei(); 					// Interrupts einschalten

	i2c_init();				// TWI initialisieren
	lm75_init();				// Temperaturfuehler (LM75) initialisieren	
	lcd_init();				// LCD initialisieren

	lcd_command(LCD_CLEAR);
	lcd_backlight(ON);
	lcd_printlc(1,1,(unsigned char *)"-->BLIT-Board<--");
	lcd_printlc(2,1,(unsigned char *)"Uwe Berger; 2008");
	lcd_wait_ms(1000);
	lcd_command(LCD_CLEAR);

	lcd_printlc(1,1,(unsigned char *)"-->BLIT-Board<--");
	lcd_printlc(2,1,(unsigned char *)"Temp..: ");
	
	while (true){
		// wenn sich i veraendert hat, i neu auf LCD ausgeben
		if (old_i != i) {
			old_i = i;
			lm75_read();

			// Formatierung/Ausgabe Temperatur
			y = 2;
			x = 9;
			if (h_byte >= 128) {			// wenn Bit7 gesetzt, dann negative Temp.
				h_byte = ~(h_byte);		// dann (lt. Datenblatt) Byte invertieren
				h_byte++;			// und plus 1
				lcd_printlr(y, x, (unsigned char *)"-");
			} else {
				lcd_printlr(y, x, (unsigned char *)"+");
			}
			x++;
			itoa(h_byte, s, 10 );			// hier steht jetzt der Vorkomma-Wert
			lcd_printlr(y, x, (unsigned char *)s);
			x++;
			if (h_byte >= 10) x++;
			if (h_byte >= 100) x++;
			lcd_printlr(y, x, (unsigned char *)",");
			x++;
			if (l_byte >= 128) {			// wenn Bit7 gesetzt, dann ,5 sonst ,0
				lcd_printlr(y, x, (unsigned char *)"5");
			} else {
				lcd_printlr(y, x, (unsigned char *)"0");
			}
			x++;
			lcd_putcharlr(y, x, 223);
			x++;
			lcd_printlr(y, x, (unsigned char *)"C");
		}
	}					
}
